@@ -56,6 +56,8 @@ class BuildClass(_base.BaseValue):
     any_var = self.ctx.new_unsolvable(node)
     for basevar in bases:
       for base in basevar.data:
+        if base.final:
+          self.ctx.errorlog.subclassing_final_class(self.ctx.vm.frames, basevar)
         if isinstance(base, ParameterizedClass):
           subst.update(
               {v.name: any_var for v in base.formal_type_parameters.values()
@@ -317,14 +319,25 @@ class PyTDClass(
     pytd_cls, decorated = decorate.process_class(pytd_cls)
     self.pytd_cls = pytd_cls
     super().__init__(name, ctx)
+    if decorate.has_decorator(
+        pytd_cls, ("typing.final", "typing_extensions.final")):
+      self.final = True
+    # Keep track of the names of final methods and instance variables.
+    self.final_members = {}
     mm = {}
     for val in pytd_cls.constants:
       if isinstance(val.type, pytd.Annotated):
         mm[val.name] = val.Replace(type=val.type.base_type)
+      elif (isinstance(val.type, pytd.GenericType) and
+            val.type.base_type.name == "typing.Final"):
+        self.final_members[val.name] = val
+        mm[val.name] = val.Replace(type=val.type.parameters[0])
       else:
         mm[val.name] = val
     for val in pytd_cls.methods:
       mm[val.name] = val
+      if val.is_final:
+        self.final_members[val.name] = val
     for val in pytd_cls.classes:
       mm[val.name.rsplit(".", 1)[-1]] = val
     if pytd_cls.metaclass is None:
@@ -554,7 +567,6 @@ class ParameterizedClass(
     # See the formal_type_parameters() property.
     self._formal_type_parameters = formal_type_parameters
     self._formal_type_parameters_loaded = False
-    self._seen_for_formal = False  # for calculating the 'formal' property
     self._hash = None  # memoized due to expensive computation
     self.official_name = self.base_cls.official_name
     if template is None:
@@ -600,14 +612,25 @@ class ParameterizedClass(
     if self._hash is None:
       if isinstance(self._formal_type_parameters,
                     abstract_utils.LazyFormalTypeParameters):
-        items = tuple(self._raw_formal_type_parameters())
+        items = self._raw_formal_type_parameters()
+        cache = False
       else:
         # Use the names of the parameter values to approximate a hash, to avoid
         # infinite recursion on recursive type annotations.
-        items = tuple((name, val.full_name)
-                      for name, val in self.formal_type_parameters.items())
-      self._hash = hash((self.base_cls, items))
-    return self._hash
+        items = []
+        cache = True
+        for name, val in self.formal_type_parameters.items():
+          # The 'is not True' check is to prevent us from incorrectly caching
+          # the hash when val.resolved == LateAnnotation._RESOLVING.
+          if val.is_late_annotation() and val.resolved is not True:  # pylint: disable=g-bool-id-comparison
+            cache = False
+          items.append((name, val.full_name))
+      hashval = hash((self.base_cls, tuple(items)))
+      if cache:
+        self._hash = hashval
+    else:
+      hashval = self._hash
+    return hashval
 
   def __contains__(self, name):
     return name in self.base_cls
@@ -630,18 +653,6 @@ class ParameterizedClass(
   def members(self):
     return self.base_cls.members
 
-  @property
-  def formal(self):
-    # We can't compute self.formal in __init__ because doing so would force
-    # evaluation of our type parameters during initialization, possibly
-    # leading to an infinite loop.
-    if self._seen_for_formal:
-      return False
-    self._seen_for_formal = True
-    formal = any(t.formal for t in self.formal_type_parameters.values())
-    self._seen_for_formal = False
-    return formal
-
   @property
   def formal_type_parameters(self):
     self._load_formal_type_parameters()
@@ -710,9 +721,11 @@ class ParameterizedClass(
     #   from typing import List
     #   print(List[str]())
     # produces 'TypeError: Type List cannot be instantiated; use list() instead'
-    # at runtime. We also disallow the builtins module because pytype represents
-    # concrete typing classes like List with their builtins equivalents.
-    return not self.is_abstract and self.module not in ("builtins", "typing")
+    # at runtime. However, pytype represents concrete typing classes like List
+    # with their builtins equivalents, so we can't distinguish between
+    # List[str]() (illegal) and list[str]() (legal in Python 3.9+); we err on
+    # the side of allowing such calls.
+    return not self.is_abstract
 
   def call(self, node, func, args, alias_map=None):
     if not self._is_callable():
@@ -884,9 +897,10 @@ class TupleClass(ParameterizedClass, mixin.HasSlots):
     return {abstract_utils.full_type_name(self, abstract_utils.T):
             self.formal_type_parameters[abstract_utils.T]}
 
-  def instantiate(self, node, container=None):
+  def _new_instance(self, container, node, args):
+    del args  # unused
     if self._instance:
-      return self._instance.to_variable(node)
+      return self._instance
     content = []
     for i in range(self.tuple_length):
       p = self.formal_type_parameters[i]
@@ -897,7 +911,12 @@ class TupleClass(ParameterizedClass, mixin.HasSlots):
         content.append(p.instantiate(self.ctx.root_node, container))
       else:
         content.append(p.instantiate(self.ctx.root_node))
-    return _instances.Tuple(tuple(content), self.ctx).to_variable(node)
+    # Note that we intentionally don't set self._instance to the new tuple,
+    # since the tuple will create and register itself with a fresh TupleClass.
+    return _instances.Tuple(tuple(content), self.ctx)
+
+  def instantiate(self, node, container=None):
+    return self._new_instance(container, node, None).to_variable(node)
 
   def _instantiate_index(self, node, index):
     if self._instance:
