@@ -367,8 +367,8 @@ class InterpreterFunction(SignedFunction):
   _function_cache = {}
 
   @classmethod
-  def make(cls, name, code, f_locals, f_globals, defaults, kw_defaults, closure,
-           annotations, ctx):
+  def make(cls, name, *, def_opcode, code, f_locals, f_globals, defaults,
+           kw_defaults, closure, annotations, ctx):
     """Get an InterpreterFunction.
 
     Things like anonymous functions and generator expressions are created
@@ -378,6 +378,7 @@ class InterpreterFunction(SignedFunction):
 
     Arguments:
       name: Function name.
+      def_opcode: The opcode for the def statement
       code: A code object.
       f_locals: The locals used for name resolution.
       f_globals: The globals used for name resolution.
@@ -417,16 +418,18 @@ class InterpreterFunction(SignedFunction):
                (dict(enumerate(defaults)), None),
                (dict(enumerate(closure or ())), None)))
     if key not in cls._function_cache:
-      cls._function_cache[key] = cls(name, code, f_locals, f_globals, defaults,
-                                     kw_defaults, closure, annotations,
+      cls._function_cache[key] = cls(name, def_opcode, code, f_locals,
+                                     f_globals, defaults, kw_defaults, closure,
+                                     annotations,
                                      overloads, ctx)
     return cls._function_cache[key]
 
-  def __init__(self, name, code, f_locals, f_globals, defaults, kw_defaults,
-               closure, annotations, overloads, ctx):
+  def __init__(self, name, def_opcode, code, f_locals, f_globals, defaults,
+               kw_defaults, closure, annotations, overloads, ctx):
     log.debug("Creating InterpreterFunction %r for %r", name, code.co_name)
     self.bound_class = _function_base.BoundInterpreterFunction
     self.doc = code.co_consts[0] if code.co_consts else None
+    self.def_opcode = def_opcode
     self.code = code
     self.f_globals = f_globals
     self.f_locals = f_locals
@@ -445,6 +448,7 @@ class InterpreterFunction(SignedFunction):
       self.nonstararg_count += self.code.co_kwonlyargcount
     signature = self._build_signature(name, annotations)
     super().__init__(signature, ctx)
+    self._check_annotations(name, annotations)
     self._update_signature_scope()
     self.last_frame = None  # for BuildClass
     self._store_call_records = False
@@ -461,6 +465,13 @@ class InterpreterFunction(SignedFunction):
     yield
     self._store_call_records = old
 
+  def _check_annotations(self, name, annotations):
+    """Validate function annotations."""
+    for ann in annotations.values():
+      if isinstance(ann, _typing.FinalAnnotation):
+        self.ctx.errorlog.invalid_final_type(
+            self.ctx.vm.simple_stack(self.def_opcode))
+
   def _build_signature(self, name, annotations):
     """Build a function.Signature object representing this function."""
     vararg_name = None
