@@ -4,7 +4,7 @@ import logging
 import typing
 from typing import Mapping, Tuple, Type, Union as _Union
 
-from pytype import utils
+from pytype import datatypes
 from pytype.abstract import _base
 from pytype.abstract import _classes
 from pytype.abstract import _instance_base
@@ -213,6 +213,11 @@ class AnnotationContainer(AnnotationClass):
           # Don't report this error again.
           inner = (self.ctx.convert.unsolvable,)
           self.ctx.errorlog.not_indexable(self.ctx.vm.frames, self.name)
+      # Check for a misused Final annotation
+      if any(isinstance(val, FinalAnnotation) for val in inner):
+        self.ctx.errorlog.invalid_final_type(self.ctx.vm.frames)
+        inner = [val.annotation if isinstance(val, FinalAnnotation) else val
+                 for val in inner]
     return inner
 
   def _build_value(self, node, inner, ellipses):
@@ -416,11 +421,11 @@ class Union(_base.BaseValue, mixin.NestedAnnotation, mixin.HasSlots):
     assert options
     self.options = list(options)
     self.cls = self._get_class()
-    self.formal = any(t.formal for t in self.options)
+    self._printing = False
+    self._instance_cache = {}
     mixin.NestedAnnotation.init_mixin(self)
     mixin.HasSlots.init_mixin(self)
     self.set_slot("__getitem__", self.getitem_slot)
-    self._printing = False
 
   def __repr__(self):
     if self._printing:  # recursion detected
@@ -440,16 +445,13 @@ class Union(_base.BaseValue, mixin.NestedAnnotation, mixin.HasSlots):
     return not self == other
 
   def __hash__(self):
-    return hash(tuple(self.options))
+    # Use the names of the parameter values to approximate a hash, to avoid
+    # infinite recursion on recursive type annotations.
+    return hash(tuple(o.full_name for o in self.options))
 
   def _unique_parameters(self):
     return [o.to_variable(self.ctx.root_node) for o in self.options]
 
-  def _get_type_params(self):
-    params = self.ctx.annotation_utils.get_type_parameters(self)
-    params = [x.full_name for x in params]
-    return utils.unique_list(params)
-
   def _get_class(self):
     classes = {o.cls for o in self.options}
     if len(classes) > 1:
@@ -460,26 +462,41 @@ class Union(_base.BaseValue, mixin.NestedAnnotation, mixin.HasSlots):
   def getitem_slot(self, node, slice_var):
     """Custom __getitem__ implementation."""
     slice_content = abstract_utils.maybe_extract_tuple(slice_var)
-    params = self._get_type_params()
+    params = self.ctx.annotation_utils.get_type_parameters(self)
+    num_params = len({x.name for x in params})
     # Check that we are instantiating all the unbound type parameters
-    if len(params) != len(slice_content):
+    if num_params != len(slice_content):
       self.ctx.errorlog.wrong_annotation_parameter_count(
           self.ctx.vm.frames, self, [v.data[0] for v in slice_content],
-          len(params))
+          num_params)
       return node, self.ctx.new_unsolvable(node)
-    concrete = []
-    for var in slice_content:
-      value = var.data[0]
-      concrete.append(
-          value.instantiate(node, container=abstract_utils.DUMMY_CONTAINER))
-    substs = [dict(zip(params, concrete))]
-    new = self.ctx.annotation_utils.sub_one_annotation(node, self, substs)
+    concrete = (
+        var.data[0].instantiate(node, container=abstract_utils.DUMMY_CONTAINER)
+        for var in slice_content)
+    subst = datatypes.AliasingDict()
+    for p in params:
+      for k in subst:
+        if k == p.name or k.endswith(f".{p.name}"):
+          subst.add_alias(p.full_name, k)
+          break
+      else:
+        subst[p.full_name] = next(concrete)
+    new = self.ctx.annotation_utils.sub_one_annotation(node, self, [subst])
     return node, new.to_variable(node)
 
   def instantiate(self, node, container=None):
     var = self.ctx.program.NewVariable()
     for option in self.options:
-      var.PasteVariable(option.instantiate(node, container), node)
+      k = (node, container, option)
+      if k in self._instance_cache:
+        if self._instance_cache[k] is None:
+          self._instance_cache[k] = self.ctx.new_unsolvable(node)
+        instance = self._instance_cache[k]
+      else:
+        self._instance_cache[k] = None
+        instance = option.instantiate(node, container)
+        self._instance_cache[k] = instance
+      var.PasteVariable(instance, node)
     return var
 
   def call(self, node, func, args, alias_map=None):
@@ -515,6 +532,8 @@ class LateAnnotation:
   Use `x.is_late_annotation()` to check whether x is a late annotation.
   """
 
+  _RESOLVING = object()
+
   def __init__(self, expr, stack, ctx):
     self.expr = expr
     self.stack = stack
@@ -531,6 +550,35 @@ class LateAnnotation:
         set(LateAnnotation.__dict__) |
         set(super().__getattribute__("__dict__")))
 
+  def flatten_expr(self):
+    """Flattens the expression into a legal variable name if necessary.
+
+    Pytype stores parameterized recursive types in intermediate variables. If
+    self is such a type, this method flattens self.expr into a string that can
+    serve as a variable name. For example, 'MyRecursiveAlias[int, str]' is
+    flattened into '_MyRecursiveAlias_LBAR_int_COMMA_str_RBAR'.
+
+    Returns:
+      If self is a parameterized recursive type, a flattened version of
+      self.expr that is a legal variable name. Otherwise, self.expr unchanged.
+    """
+    if "[" in self.expr and self.is_recursive():
+      # _DOT and _RBAR have no trailing underscore because they precede names
+      # that we already prefix an underscore to.
+      return "_" + self.expr.replace(".", "_DOT").replace(
+          "[", "_LBAR_").replace("]", "_RBAR").replace(", ", "_COMMA_")
+    return self.expr
+
+  def unflatten_expr(self):
+    """Unflattens a flattened expression."""
+    if "_LBAR_" in self.expr:
+      mod, dot, rest = self.expr.rpartition(".")
+      # The [1:] slicing and trailing underscore in _DOT_ are to get rid of
+      # leading underscores added when flattening.
+      return mod + dot + rest[1:].replace("_DOT_", ".").replace(
+          "_LBAR_", "[").replace("_RBAR", "]").replace("_COMMA_", ", ")
+    return self.expr
+
   def __repr__(self):
     return "LateAnnotation(%r, resolved=%r)" % (
         self.expr, self._type if self.resolved else None)
@@ -558,7 +606,10 @@ class LateAnnotation:
     """Resolve the late annotation."""
     if self.resolved:
       return
-    self.resolved = True
+    # Sets resolved to a truthy value distinguishable from True so that
+    # 'if self.resolved' is True when self is partially resolved, but code that
+    # really needs to tell partially and fully resolved apart can do so.
+    self.resolved = LateAnnotation._RESOLVING
     var, errorlog = abstract_utils.eval_expr(self.ctx, node, f_globals,
                                              f_locals, self.expr)
     if errorlog:
@@ -624,3 +675,17 @@ class LateAnnotation:
       if isinstance(t, mixin.NestedAnnotation):
         stack.extend(child for _, child in t.get_inner_types())
     return False
+
+
+class FinalAnnotation(_base.BaseValue):
+  """Container for a Final annotation."""
+
+  def __init__(self, annotation, ctx):
+    super().__init__("FinalAnnotation", ctx)
+    self.annotation = annotation
+
+  def __repr__(self):
+    return f"Final[{self.annotation}]"
+
+  def instantiate(self, node, container=None):
+    return self.to_variable(node)
