@@ -1,13 +1,25 @@
 """Utilities for inline type annotations."""
 
 import collections
+import dataclasses
 import itertools
 
+from typing import Any
+
 from pytype import utils
 from pytype.abstract import abstract
 from pytype.abstract import abstract_utils
 from pytype.abstract import mixin
+from pytype.overlays import typed_dict
 from pytype.overlays import typing_overlay
+from pytype.pytd import pytd_utils
+
+
+@dataclasses.dataclass
+class AnnotatedValue:
+  typ: Any
+  value: Any
+  final: bool = False
 
 
 class AnnotationUtils(utils.ContextWeakrefMixin):
@@ -21,6 +33,27 @@ class AnnotationUtils(utils.ContextWeakrefMixin):
               for name, annot in annotations.items()}
     return annotations
 
+  def _get_type_parameter_subst(self, node, annot, substs, instantiate_unbound):
+    """Helper for sub_one_annotation."""
+    assert isinstance(annot, abstract.TypeParameter)
+    # We use the given substitutions to bind the annotation if
+    # (1) every subst provides at least one binding, and
+    # (2) none of the bindings are ambiguous, and
+    # (3) at least one binding is non-empty.
+    if all(annot.full_name in subst and subst[annot.full_name].bindings
+           for subst in substs):
+      vals = sum((subst[annot.full_name].data for subst in substs), [])
+    else:
+      vals = None
+    if (vals is None or
+        any(isinstance(v, abstract.AMBIGUOUS) for v in vals) or
+        all(isinstance(v, abstract.Empty) for v in vals)):
+      if instantiate_unbound:
+        vals = annot.instantiate(node).data
+      else:
+        vals = [annot]
+    return self.ctx.convert.merge_classes(vals)
+
   def sub_one_annotation(self, node, annot, substs, instantiate_unbound=True):
     """Apply type parameter substitutions to an annotation."""
     # We push annotations onto 'stack' and move them to the 'done' stack as they
@@ -42,9 +75,16 @@ class AnnotationUtils(utils.ContextWeakrefMixin):
         if cur.is_late_annotation() and any(t[0] == cur for t in stack):
           # We've found a recursive type. We generate a LateAnnotation as a
           # placeholder for the substituted type.
-          late_annot = abstract.LateAnnotation(cur.expr, cur.stack, cur.ctx)
-          late_annotations[cur] = late_annot
-          done.append(late_annot)
+          if cur not in late_annotations:
+            param_strings = []
+            for t in utils.unique_list(self.get_type_parameters(cur)):
+              s = pytd_utils.Print(self._get_type_parameter_subst(
+                  node, t, substs, instantiate_unbound).get_instance_type(node))
+              param_strings.append(s)
+            expr = "%s[%s]" % (cur.expr, ", ".join(param_strings))
+            late_annot = abstract.LateAnnotation(expr, cur.stack, cur.ctx)
+            late_annotations[cur] = late_annot
+          done.append(late_annotations[cur])
         elif inner_type_keys is None:
           keys, vals = zip(*cur.get_inner_types())
           stack.append((cur, keys))
@@ -57,27 +97,19 @@ class AnnotationUtils(utils.ContextWeakrefMixin):
           if cur in late_annotations:
             # If we've generated a LateAnnotation placeholder for cur's
             # substituted type, replace it now with the real type.
-            late_annotations[cur].set_type(done_annot)
-          done.append(cur.replace(inner_types))
+            late_annot = late_annotations.pop(cur)
+            late_annot.set_type(done_annot)
+            if "[" in late_annot.expr:
+              if self.ctx.vm.late_annotations is None:
+                self.ctx.vm.flatten_late_annotation(
+                    node, late_annot, self.ctx.vm.frame.f_globals)
+              else:
+                self.ctx.vm.late_annotations[
+                    late_annot.expr.split("[", 1)[0]].append(late_annot)
+          done.append(done_annot)
       else:
-        assert isinstance(cur, abstract.TypeParameter)
-        # We use the given substitutions to bind the annotation if
-        # (1) every subst provides at least one binding, and
-        # (2) none of the bindings are ambiguous, and
-        # (3) at least one binding is non-empty.
-        if all(cur.full_name in subst and subst[cur.full_name].bindings
-               for subst in substs):
-          vals = sum((subst[cur.full_name].data for subst in substs), [])
-        else:
-          vals = None
-        if (vals is None or
-            any(isinstance(v, abstract.AMBIGUOUS) for v in vals) or
-            all(isinstance(v, abstract.Empty) for v in vals)):
-          if instantiate_unbound:
-            vals = cur.instantiate(node).data
-          else:
-            vals = [cur]
-        done.append(self.ctx.convert.merge_classes(vals))
+        done.append(self._get_type_parameter_subst(
+            node, cur, substs, instantiate_unbound))
     assert len(done) == 1
     return done[0]
 
@@ -88,17 +120,23 @@ class AnnotationUtils(utils.ContextWeakrefMixin):
       for _, typ in annot.get_inner_types():
         yield from self.get_late_annotations(typ)
 
-  def remove_late_annotations(self, annot):
+  def remove_late_annotations(self, annot, seen=None):
     """Replace unresolved late annotations with unsolvables."""
+    if seen is None:
+      seen = {annot}
+    elif annot in seen:
+      return annot
+    else:
+      seen.add(annot)
     if annot.is_late_annotation() and not annot.resolved:
       return self.ctx.convert.unsolvable
     elif isinstance(annot, mixin.NestedAnnotation):
-      inner_types = [(key, self.remove_late_annotations(val))
+      inner_types = [(key, self.remove_late_annotations(val, seen))
                      for key, val in annot.get_inner_types()]
       return annot.replace(inner_types)
     return annot
 
-  def add_scope(self, annot, types, module):
+  def add_scope(self, annot, types, module, seen=None):
     """Add scope for type parameters.
 
     In original type class, all type parameters that should be added a scope
@@ -108,24 +146,25 @@ class AnnotationUtils(utils.ContextWeakrefMixin):
       annot: The type class.
       types: A type name list that should be added a scope.
       module: Module name.
+      seen: Already seen types.
 
     Returns:
       The type with fresh type parameters that have been added the scope.
     """
+    if seen is None:
+      seen = {annot}
+    elif annot in seen:
+      return annot
+    else:
+      seen.add(annot)
     if isinstance(annot, abstract.TypeParameter):
       if annot.name in types:
         new_annot = annot.copy()
         new_annot.module = module
         return new_annot
       return annot
-    elif isinstance(annot, abstract.TupleClass):
-      params = {}
-      for name, param in annot.formal_type_parameters.items():
-        params[name] = self.add_scope(param, types, module)
-      return abstract.TupleClass(annot.base_cls, params, self.ctx,
-                                 annot.template)
     elif isinstance(annot, mixin.NestedAnnotation):
-      inner_types = [(key, self.add_scope(typ, types, module))
+      inner_types = [(key, self.add_scope(typ, types, module, seen))
                      for key, typ in annot.get_inner_types()]
       return annot.replace(inner_types)
     return annot
@@ -143,7 +182,7 @@ class AnnotationUtils(utils.ContextWeakrefMixin):
     seen = seen or set()
     if annot in seen:
       return []
-    if isinstance(annot, abstract.ParameterizedClass):
+    if isinstance(annot, mixin.NestedAnnotation):
       # We track parameterized classes to avoid recursion errors when a class
       # contains itself.
       seen = seen | {annot}
@@ -212,7 +251,8 @@ class AnnotationUtils(utils.ContextWeakrefMixin):
     """Convert a (name, raw_annot) list to a {name: annotation} dict."""
     annotations = {}
     for name, t in annotations_list:
-      if t is None:
+      if t is None or t == self.ctx.convert.ellipsis:
+        # '...' is an experimental "inferred type": see b/213607272.
         continue
       annot = self._process_one_annotation(node, t, name,
                                            self.ctx.vm.simple_stack())
@@ -271,22 +311,28 @@ class AnnotationUtils(utils.ContextWeakrefMixin):
         name,
         self.ctx.vm.simple_stack(),
         allowed_type_params=allowed_type_params)
+    orig_typ = typ
+    if isinstance(typ, abstract.FinalAnnotation):
+      typ = typ.annotation
+    elif isinstance(typ, typing_overlay.Final):
+      typ = self.ctx.convert.unsolvable
     if typ.formal:
-      resolved_type = self.sub_one_annotation(node, typ, substs,
-                                              instantiate_unbound=False)
-      _, value = self.init_annotation(node, name, resolved_type)
-    else:
-      _, value = self.init_annotation(node, name, typ)
-    return typ, value
+      typ = self.sub_one_annotation(node, typ, substs,
+                                    instantiate_unbound=False)
+    _, value = self.init_annotation(node, name, typ)
+    return orig_typ, value
 
   def apply_annotation(self, node, op, name, value):
     """If there is an annotation for the op, return its value."""
     assert op is self.ctx.vm.frame.current_opcode
     if op.code.co_filename != self.ctx.vm.filename:
-      return None, value
+      return AnnotatedValue(None, value)
     if not op.annotation:
-      return None, value
+      return AnnotatedValue(None, value)
     annot = op.annotation
+    if annot == "...":
+      # Experimental "inferred type": see b/213607272.
+      return AnnotatedValue(None, value)
     frame = self.ctx.vm.frame
     with self.ctx.vm.generate_late_annotations(self.ctx.vm.simple_stack()):
       var, errorlog = abstract_utils.eval_expr(self.ctx, node, frame.f_globals,
@@ -294,7 +340,18 @@ class AnnotationUtils(utils.ContextWeakrefMixin):
     if errorlog:
       self.ctx.errorlog.invalid_annotation(
           self.ctx.vm.frames, annot, details=errorlog.details)
-    return self.extract_and_init_annotation(node, name, var)
+    typ, annot_val = self.extract_and_init_annotation(node, name, var)
+    if isinstance(typ, typing_overlay.Final):
+      # return the original value, we want type inference here.
+      return AnnotatedValue(None, value, final=True)
+    elif isinstance(typ, abstract.FinalAnnotation):
+      return AnnotatedValue(typ.annotation, annot_val, final=True)
+    elif isinstance(typ, typed_dict.TypedDictClass):
+      # We do not want to instantiate a TypedDict annotation if we have a
+      # concrete value.
+      return AnnotatedValue(typ, value)
+    else:
+      return AnnotatedValue(typ, annot_val)
 
   def extract_annotation(
       self, node, var, name, stack, allowed_type_params=None):
@@ -437,6 +494,8 @@ class AnnotationUtils(utils.ContextWeakrefMixin):
     elif isinstance(annotation, (abstract.Class,
                                  abstract.AMBIGUOUS_OR_EMPTY,
                                  abstract.TypeParameter,
+                                 abstract.FinalAnnotation,
+                                 typing_overlay.Final,
                                  typing_overlay.NoReturn)):
       return annotation
     else:
