@@ -25,7 +25,8 @@ class ValselfTest(test_base.UnitTest):
 
   def setUp(self):
     super().setUp()
-    options = config.Options.create(python_version=self.python_version)
+    options = config.Options.create(python_version=self.python_version,
+                                    color="never")
     self.ctx = context.Context(errors.ErrorLog(), options,
                                load_pytd.Loader(options))
     self.node = self.ctx.root_node
