@@ -147,8 +147,6 @@ def add_basic_options(o):
 FEATURE_FLAGS = [
     ("--use-enum-overlay", False,
      "Use the enum overlay for more precise enum checking."),
-    ("--allow-recursive-types", False,
-     "Allow recursive type definitions."),
     ("--build-dict-literals-from-kwargs", False,
      "Build dict literals from dict(k=v, ...) calls."),
     ("--strict_namedtuple_checks", False,
@@ -285,6 +283,10 @@ def add_infrastructure_options(o):
       "-Z", "--quick", action="store_true",
       dest="quick", default=None,
       help=("Only do an approximation."))
+  o.add_argument(
+      "--color", action="store", choices=["always", "auto", "never"],
+      default="auto",
+      help="Choose never to disable color in the shell output.")
 
 
 def add_debug_options(o):
@@ -572,6 +574,19 @@ class Postprocessor:
       self.error("Not allowed with --no-report-errors", "output-errors-csv")
     self.output_options.output_errors_csv = output_errors_csv
 
+  def _store_color(self, color):
+    if color not in ("always", "auto", "never"):
+      raise ValueError(f"--color flag allows only 'always', 'auto' or 'never', "
+                       f"not {color!r}")
+
+    self.output_options.color = (
+        sys.platform in ("linux", "cygwin", "darwin") and
+        (
+            color == "always" or
+            (color == "auto" and sys.stderr.isatty())
+        )
+    )
+
   @uses(["input", "pythonpath"])
   def _store_module_name(self, module_name):
     if module_name is None:
