@@ -2,9 +2,12 @@
 """Tests for directors.py."""
 
 import sys
+import textwrap
 
+from pytype import blocks
 from pytype import directors
 from pytype import errors
+from pytype.pyc import pyc
 import unittest
 
 _TEST_FILENAME = "my_file.py"
@@ -99,6 +102,8 @@ class LineSetTest(unittest.TestCase):
 
 class DirectorTestCase(unittest.TestCase):
 
+  python_version = sys.version_info[:2]
+
   @classmethod
   def setUpClass(cls):
     super(DirectorTestCase, cls).setUpClass()
@@ -108,9 +113,17 @@ class DirectorTestCase(unittest.TestCase):
       errors._error_name(name)
 
   def _create(self, src, disable=()):
+    self.num_lines = len(src.rstrip().splitlines())
+    src = textwrap.dedent(src)
+    src_tree = directors.parse_src(src, self.python_version)
     self._errorlog = errors.ErrorLog()
-    self._director = directors.Director(src, self._errorlog, _TEST_FILENAME,
-                                        disable, sys.version_info[:2])
+    if self.python_version < (3, 8):
+      raw_code = pyc.compile_src(src, _TEST_FILENAME, self.python_version, None)
+      code = blocks.process_code(raw_code, self.python_version)
+    else:
+      code = None
+    self._director = directors.Director(
+        src_tree, self._errorlog, _TEST_FILENAME, disable, code)
 
   def _should_report(self, expected, lineno, error_name="test-error",
                      filename=_TEST_FILENAME):
@@ -150,7 +163,7 @@ class DirectorTest(DirectorTestCase):
 
   def test_utf8(self):
     self._create("""
-    x = u"abc□def\n"
+    x = u"abc□def\\n"
     """)
 
   def test_ignore_extra_characters(self):
@@ -354,75 +367,13 @@ class DirectorTest(DirectorTestCase):
         4: "str",
     }, self._director.type_comments)
 
-  def test_type_comment_on_multiline_value(self):
-    # should_be_ignored will be filtered out later by adjust_line_numbers.
-    self._create("""
-    v = [
-      ("hello",
-       "world",  # type: should_be_ignored
-
-      )
-    ]  # type: dict
-    """)
-    self.assertEqual({
-        4: "should_be_ignored",
-        7: "dict",
-    }, self._director.type_comments)
-
-  def test_type_comment_with_trailing_comma(self):
-    self._create("""
-    v = [
-      ("hello",
-       "world"
-      ),
-    ]  # type: dict
-    w = [
-      ["hello",
-       "world"
-      ],  # some comment
-    ]  # type: dict
-    """)
-    self.assertEqual({
-        6: "dict",
-        11: "dict",
-    }, self._director.type_comments)
-
-  def test_decorators(self):
-    self._create("""
-      class A:
-        '''
-        @decorator in a docstring
-        '''
-        @real_decorator
-        def f(x):
-          x = foo @ bar @ baz
-
-        @decorator(
-            x, y
-        )
-
-        def bar():
-          pass
-    """)
-    self.assertEqual({
-        7,  # real_decorator
-        14  # decorator
-    }, self._director._decorators)
-
-  def test_stacked_decorators(self):
-    self._create("""
-      @decorator(
-          x, y
-      )
-
-      @foo
-
-      class A:
-          pass
-    """)
-    self.assertEqual({
-        8  # foo
-    }, self._director._decorators)
+  def test_huge_string(self):
+    # Tests that the director doesn't choke on this huge, repetitive file.
+    src = ["x = ("]
+    for i in range(2000):
+      src.append(f"    'string{i}'")
+    src.append(")")
+    self._create("\n".join(src))
 
 
 class VariableAnnotationsTest(DirectorTestCase):
@@ -451,9 +402,6 @@ class VariableAnnotationsTest(DirectorTestCase):
     """)
     self.assertFalse(self._director.annotations)
 
-  @unittest.skip(
-      "directors._VariableAnnotation assumes a variable annotation starts at "
-      "the beginning of the line.")
   def test_multistatement_line(self):
     self._create("""
       if __random__: v1: int = 0
@@ -481,7 +429,7 @@ class VariableAnnotationsTest(DirectorTestCase):
 
   def test_multiline_annotation(self):
     self._create("""
-      v: Callable[
+      v: Callable[  # a very important comment
           [], int] = None
     """)
     lineno = 2 if sys.version_info[:2] >= (3, 8) else 3
@@ -494,7 +442,7 @@ class VariableAnnotationsTest(DirectorTestCase):
           1,
       ]
     """)
-    lineno = 2 if sys.version_info[:2] >= (3, 8) else 5
+    lineno = 2 if sys.version_info[:2] >= (3, 8) else 4
     self.assertEqual({lineno: "List[int]"}, self._director.annotations)
 
   def test_complicated_annotation(self):
@@ -518,5 +466,372 @@ class VariableAnnotationsTest(DirectorTestCase):
     self.assertEqual({3: "int"}, self._director.annotations)
 
 
+class LineNumbersTest(DirectorTestCase):
+
+  def test_type_comment_on_multiline_value(self):
+    self._create("""
+      v = [
+        ("hello",
+         "world",  # type: should_be_ignored
+
+        )
+      ]  # type: dict
+    """)
+    # The line number of STORE_NAME v changes between versions.
+    if self.python_version >= (3, 8):
+      lineno = 2
+    else:
+      lineno = 3
+    self.assertEqual({lineno: "dict"}, self._director.type_comments)
+
+  def test_type_comment_with_trailing_comma(self):
+    self._create("""
+      v = [
+        ("hello",
+         "world"
+        ),
+      ]  # type: dict
+      w = [
+        ["hello",
+         "world"
+        ],  # some comment
+      ]  # type: dict
+    """)
+    # The line numbers of STORE_NAME change between versions.
+    if self.python_version >= (3, 8):
+      v_lineno = 2
+      w_lineno = 7
+    else:
+      v_lineno = 3
+      w_lineno = 9
+    self.assertEqual({
+        v_lineno: "dict",
+        w_lineno: "dict"
+    }, self._director.type_comments)
+
+  def test_decorators(self):
+    self._create("""
+      class A:
+        '''
+        @decorator in a docstring
+        '''
+        @real_decorator
+        def f(x):
+          x = foo @ bar @ baz
+
+        @decorator(
+            x, y
+        )
+
+        def bar():
+          pass
+    """)
+    if self.python_version >= (3, 8):
+      real_decorator_lineno = 7
+      decorator_lineno = 14
+    else:
+      real_decorator_lineno = 6
+      decorator_lineno = 11
+    self.assertEqual(self._director.decorators,
+                     {real_decorator_lineno, decorator_lineno})
+
+  def test_stacked_decorators(self):
+    self._create("""
+      @decorator(
+          x, y
+      )
+
+      @foo
+
+      class A:
+          pass
+    """)
+    lineno = 8 if self.python_version >= (3, 8) else 6
+    self.assertEqual(self._director.decorators, {lineno})
+
+  def test_overload(self):
+    self._create("""
+      from typing import overload
+
+      @overload
+      def f() -> int: ...
+
+      @overload
+      def f(x: str) -> str: ...
+
+      def f(x=None):
+        return 0 if x is None else x
+    """)
+    self.assertEqual(self._director.decorators, {5, 8})
+
+
+class DisableDirectivesTest(DirectorTestCase):
+
+  def assertDisables(self, *disable_lines, error_class=None, disables=None):
+    assert not (error_class and disables)
+    error_class = error_class or "wrong-arg-types"
+    disables = disables or self._director._disables[error_class]
+    for i in range(self.num_lines):
+      lineno = i+1
+      if lineno in disable_lines:
+        self.assertIn(lineno, disables)
+      else:
+        self.assertNotIn(lineno, disables)
+
+  def test_basic(self):
+    self._create("""
+      toplevel(
+          a, b, c, d)  # pytype: disable=wrong-arg-types
+    """)
+    if self.python_version >= (3, 8):
+      self.assertDisables(2)
+    else:
+      self.assertDisables(3)
+
+  def test_nested(self):
+    self._create("""
+      toplevel(
+          nested())  # pytype: disable=wrong-arg-types
+    """)
+    if self.python_version >= (3, 8):
+      self.assertDisables(2, 3)
+    else:
+      self.assertDisables(3)
+
+  def test_multiple_nested(self):
+    self._create("""
+      toplevel(
+        nested1(),
+        nested2(),  # pytype: disable=wrong-arg-types
+        nested3())
+    """)
+    if self.python_version >= (3, 8):
+      self.assertDisables(2, 4)
+    else:
+      self.assertDisables(4)
+
+  def test_multiple_toplevel(self):
+    self._create("""
+      toplevel1()
+      toplevel2()  # pytype: disable=wrong-arg-types
+      toplevel3()
+    """)
+    self.assertDisables(3)
+
+  def test_deeply_nested(self):
+    self._create("""
+      toplevel(
+        nested1(),
+        nested2(
+          deeply_nested1(),  # pytype: disable=wrong-arg-types
+          deeply_nested2()),
+        nested3())
+    """)
+    if self.python_version >= (3, 8):
+      self.assertDisables(2, 4, 5)
+    else:
+      self.assertDisables(5)
+
+  def test_non_toplevel(self):
+    self._create("""
+      x = [
+        f("oops")  # pytype: disable=wrong-arg-types
+      ]
+    """)
+    if self.python_version >= (3, 8):
+      self.assertDisables(2, 3)
+    else:
+      self.assertDisables(3)
+
+  def test_non_toplevel_bad_annotation(self):
+    self._create("""
+      x: list[int] = [
+        f(
+            "oops")]  # pytype: disable=annotation-type-mismatch
+    """)
+    if self.python_version >= (3, 8):
+      self.assertDisables(2, error_class="annotation-type-mismatch")
+    else:
+      self.assertDisables(4, error_class="annotation-type-mismatch")
+
+  def test_trailing_parenthesis(self):
+    self._create("""
+      toplevel(
+          a, b, c, d,
+      )  # pytype: disable=wrong-arg-types
+    """)
+    if self.python_version >= (3, 8):
+      self.assertDisables(2)
+    else:
+      self.assertDisables(3)
+
+  def test_multiple_bytecode_blocks(self):
+    self._create("""
+      def f():
+        call(a, b, c, d)
+      def g():
+        call(a, b, c, d)  # pytype: disable=wrong-arg-types
+    """)
+    self.assertDisables(5)
+
+  def test_compare(self):
+    self._create("""
+      import datetime
+      def f(right: datetime.date):
+        left = datetime.datetime(1, 1, 1, 1)
+        return left < right  # pytype: disable=wrong-arg-types
+    """)
+    self.assertDisables(5)
+
+  def test_iterate(self):
+    self._create("""
+      class Foo:
+        def __iter__(self, too, many, args):
+          pass
+      foo = Foo()
+      for x in foo:  # pytype: disable=missing-parameter
+        print(x)
+    """)
+    self.assertDisables(6, error_class="missing-parameter")
+
+  def test_subscript(self):
+    self._create("""
+      class Foo:
+        def __getitem__(self, too, many, args):
+          pass
+      x = Foo()
+      x['X']  # pytype: disable=missing-parameter
+    """)
+    self.assertDisables(6, error_class="missing-parameter")
+
+  def test_attrs(self):
+    self._create("""
+      import attr
+      def converter(x):
+        return []
+      @attr.s
+      class Foo:
+        x = attr.ib(
+          converter=converter, factory=list, type=dict[str, str]
+        )  # pytype: disable=annotation-type-mismatch
+    """)
+    if self.python_version >= (3, 8):
+      self.assertDisables(7, error_class="annotation-type-mismatch")
+    else:
+      self.assertDisables(8, error_class="annotation-type-mismatch")
+
+  def test_return(self):
+    self._create("""
+       def f(x):
+         return x
+       def g() -> int:
+         return f(
+             "oops")  # pytype: disable=bad-return-type
+    """)
+    if self.python_version >= (3, 8):
+      self.assertDisables(5, error_class="bad-return-type")
+    else:
+      self.assertDisables(6, error_class="bad-return-type")
+
+  def test_if(self):
+    self._create("""
+      if (__random__ and
+          name_error and  # pytype: disable=name-error
+          __random__):
+        pass
+    """)
+    self.assertDisables(3, error_class="name-error")
+
+  def test_unsupported(self):
+    self._create("""
+      x = [
+        "something_unsupported"
+      ]  # pytype: disable=not-supported-yet
+    """)
+    if self.python_version >= (3, 8):
+      self.assertDisables(2, error_class="not-supported-yet")
+    else:
+      self.assertDisables(3, error_class="not-supported-yet")
+
+  def test_range(self):
+    self._create("""
+      f(
+        # pytype: disable=attribute-error
+        a.nonsense,
+        b.nonsense,
+        # pytype: enable=attribute-error
+      )
+    """)
+    self.assertDisables(3, 4, 5, error_class="attribute-error")
+
+  def test_ignore(self):
+    # We have no idea if the '# type: ignore' is for the list construction, the
+    # function call, or the function argument, so we apply it to all of them.
+    self._create("""
+      x = [
+        some_bad_function(
+            "some bad arg")]  # type: ignore
+    """)
+    if self.python_version >= (3, 8):
+      self.assertDisables(2, 3, 4, disables=self._director.ignore)
+    else:
+      self.assertDisables(4, disables=self._director.ignore)
+
+  def test_ignore_range(self):
+    self._create("""
+      x = [
+        # type: ignore
+        "oops"
+      ]
+    """)
+    self.assertDisables(3, 4, 5, disables=self._director.ignore)
+
+  def test_with_and_backslash_continuation(self):
+    self._create("""
+      with foo("a",
+               "b"), \\
+           bar("c",
+               "d"),  \\
+           baz("e"):  # pytype: disable=wrong-arg-types
+        pass
+    """)
+    if self.python_version >= (3, 8):
+      self.assertDisables(2, 6)
+    else:
+      self.assertDisables(6)
+
+  def test_not_instantiable(self):
+    self._create("""
+      x = [
+        A(
+      )]  # pytype: disable=not-instantiable
+    """)
+    if self.python_version >= (3, 8):
+      self.assertDisables(2, 3, error_class="not-instantiable")
+    else:
+      self.assertDisables(3, error_class="not-instantiable")
+
+  def test_unsupported_operands_in_call(self):
+    self._create("""
+      some_func(
+        x < y)  # pytype: disable=unsupported-operands
+    """)
+    if self.python_version >= (3, 8):
+      self.assertDisables(2, 3, error_class="unsupported-operands")
+    else:
+      self.assertDisables(3, error_class="unsupported-operands")
+
+  def test_unsupported_operands_in_assignment(self):
+    self._create("""
+      x["wrong key type"] = (
+        some_call(),
+        "oops")  # pytype: disable=unsupported-operands
+    """)
+    if self.python_version >= (3, 8):
+      self.assertDisables(2, error_class="unsupported-operands")
+    else:
+      self.assertDisables(4, error_class="unsupported-operands")
+
+
 if __name__ == "__main__":
   unittest.main()
