@@ -52,6 +52,13 @@ _ELLIPSIS = object()
 
 _FuncT = TypeVar("_FuncT", bound=Callable)
 
+_STYLE_BRIGHT = "\x1b[1m"
+_STYLE_RESET_ALL = "\x1b[0m"
+_FORE_RED = "\x1b[31m"
+_FORE_RESET = "\x1b[39m"
+_COLOR_ERROR_NAME_TEMPLATE = (_STYLE_BRIGHT + _FORE_RED + "%s" + _FORE_RESET +
+                              _STYLE_RESET_ALL)
+
 
 def _error_name(name) -> Callable[[_FuncT], _FuncT]:
   """Decorate a function so that it binds the current error name."""
@@ -109,10 +116,13 @@ def _make_traceback_str(frames):
 def _dedup_opcodes(stack):
   """Dedup the opcodes in a stack of frames."""
   deduped_stack = []
+  if len(stack) > 1:
+    stack = [x for x in stack if not x.skip_in_tracebacks]
   for frame in stack:
-    if frame.current_opcode and (
-        not deduped_stack or
-        frame.current_opcode.line != deduped_stack[-1].current_opcode.line):
+    if frame.current_opcode:
+      if deduped_stack and (
+          frame.current_opcode.line == deduped_stack[-1].current_opcode.line):
+        continue
       # We can have consecutive opcodes with the same line number due to, e.g.,
       # a set comprehension. The first opcode we encounter is the one with the
       # real method name, whereas the second's method name is something like
@@ -308,10 +318,15 @@ class Error:
       return ""
 
   def __str__(self):
+    return self.as_string()
+
+  def as_string(self, *, color=False):
+    """Format the error as a friendly string, optionally with shell coloring."""
     pos = self._position()
     if pos:
       pos += ": "
-    text = "%s%s [%s]" % (pos, self._message.replace("\n", "\n  "), self._name)
+    name = _COLOR_ERROR_NAME_TEMPLATE % (self._name,) if color else self._name
+    text = "%s%s [%s]" % (pos, self._message.replace("\n", "\n  "), name)
     if self._details:
       text += "\n  " + self._details.replace("\n", "\n  ")
     if self._traceback:
@@ -420,9 +435,9 @@ class ErrorLogBase:
              error._message,
              details])
 
-  def print_to_file(self, fi):
+  def print_to_file(self, fi, *, color=False):
     for error in self.unique_sorted_errors():
-      print(error, file=fi)
+      print(error.as_string(color=color), file=fi)
 
   def unique_sorted_errors(self):
     """Gets the unique errors in this log, sorted on filename and lineno."""
@@ -459,8 +474,8 @@ class ErrorLogBase:
   def _sorted_errors(self):
     return sorted(self._errors, key=lambda x: (x.filename or "", x.lineno))
 
-  def print_to_stderr(self):
-    self.print_to_file(sys.stderr)
+  def print_to_stderr(self, *, color=True):
+    self.print_to_file(sys.stderr, color=color)
 
   def __str__(self):
     f = io.StringIO()
@@ -496,16 +511,14 @@ class ErrorLog(ErrorLogBase):
 
   def _print_as_expected_type(self, t: abstract.BaseValue, instance=None):
     """Print abstract value t as a pytd type."""
-    if t.is_late_annotation():
-      return typing.cast(abstract.LateAnnotation, t).expr
-    elif isinstance(t, abstract.Union):
-      return self._join_printed_types(self._print_as_expected_type(o)
-                                      for o in t.options)
-    elif isinstance(t, (abstract.Unknown, abstract.Unsolvable,
-                        abstract.Class)):
+    if isinstance(t, (abstract.Unknown, abstract.Unsolvable,
+                      abstract.Class)) or t.is_late_annotation():
       with t.ctx.pytd_convert.set_output_mode(
           t.ctx.pytd_convert.OutputMode.DETAILED):
         return self._pytd_print(t.get_instance_type(instance=instance))
+    elif isinstance(t, abstract.Union):
+      return self._join_printed_types(self._print_as_expected_type(o)
+                                      for o in t.options)
     elif abstract_utils.is_concrete(t):
       return re.sub(r"(\\n|\s)+", " ",
                     typing.cast(mixin.PythonConstant, t).str_of_constant(
@@ -558,14 +571,18 @@ class ErrorLog(ErrorLogBase):
 
   def _print_as_function_def(self, fn):
     assert isinstance(fn, abstract.Function)
-    conv = fn.ctx.pytd_convert
+    convert = fn.ctx.pytd_convert
     name = fn.name.rsplit(".", 1)[-1]  # We want `def bar()` not `def Foo.bar()`
-    return pytd_utils.Print(conv.value_to_pytd_def(fn.ctx.root_node, fn, name))
+    with convert.set_output_mode(convert.OutputMode.DETAILED):
+      pytd_def = convert.value_to_pytd_def(fn.ctx.root_node, fn, name)
+    return pytd_utils.Print(pytd_def)
 
   def _print_protocol_error(self, error):
     """Pretty-print the matcher.ProtocolError instance."""
-    left = self._pytd_print(error.left_type.get_instance_type())
-    protocol = self._pytd_print(error.other_type.get_instance_type())
+    convert = error.left_type.ctx.pytd_convert
+    with convert.set_output_mode(convert.OutputMode.DETAILED):
+      left = self._pytd_print(error.left_type.get_instance_type())
+      protocol = self._pytd_print(error.other_type.get_instance_type())
     if isinstance(error, matcher.ProtocolMissingAttributesError):
       missing = ", ".join(sorted(error.missing))
       return (f"Attributes of protocol {protocol} are not implemented on "
@@ -585,8 +602,9 @@ class ErrorLog(ErrorLogBase):
                 f">> {protocol} expects:\n{expected}\n\n"
                 f">> {left} defines:\n{actual}")
       else:
-        actual = self._pytd_print(error.actual_type.to_type())
-        expected = self._pytd_print(error.expected_type.to_type())
+        with convert.set_output_mode(convert.OutputMode.DETAILED):
+          actual = self._pytd_print(error.actual_type.to_type())
+          expected = self._pytd_print(error.expected_type.to_type())
         return (f"Attribute {error.attribute_name} of protocol {protocol} has "
                 f"wrong type in {left}: expected {expected}, got {actual}")
 
@@ -1302,6 +1320,52 @@ class ErrorLog(ErrorLogBase):
       err_msg = f"TypedDict {obj.class_name} requires all keys to be strings"
     self.error(stack, err_msg)
 
+  @_error_name("final-error")
+  def _overriding_final(self, stack, cls, base, name, *, is_method, details):
+    desc = "method" if is_method else "class attribute"
+    msg = (f"Class {cls.name} overrides final {desc} {name}, "
+           f"defined in base class {base.name}")
+    self.error(stack, msg, details=details)
+
+  def overriding_final_method(self, stack, cls, base, name, details=None):
+    self._overriding_final(stack, cls, base, name, details=details,
+                           is_method=True)
+
+  def overriding_final_attribute(self, stack, cls, base, name, details=None):
+    self._overriding_final(stack, cls, base, name, details=details,
+                           is_method=False)
+
+  @_error_name("final-error")
+  def assigning_to_final(self, stack, name, local):
+    """Attempting to reassign a variable annotated with Final."""
+    obj = "variable" if local else "attribute"
+    err_msg = f"Assigning to {obj} {name}, which was annotated with Final"
+    self.error(stack, err_msg)
+
+  @_error_name("final-error")
+  def subclassing_final_class(self, stack, base_var, details=None):
+    base_cls = self._join_printed_types(
+        self._print_as_expected_type(t) for t in base_var.data)
+    self.error(stack, "Cannot subclass final class: %s" % base_cls,
+               details=details, keyword=base_cls)
+
+  @_error_name("final-error")
+  def bad_final_decorator(self, stack, obj, details=None):
+    name = getattr(obj, "name", None)
+    if not name:
+      typ = self._print_as_expected_type(obj)
+      name = f"object of type {typ}"
+    msg = f"Cannot apply @final decorator to {name}"
+    details = "@final can only be applied to classes and methods."
+    self.error(stack, msg, details=details)
+
+  @_error_name("final-error")
+  def invalid_final_type(self, stack, details=None):
+    msg = "Invalid use of typing.Final"
+    details = ("Final may only be used as the outermost type in assignments "
+               "or variable annotations.")
+    self.error(stack, msg, details=details)
+
 
 def get_error_names_set():
   return _ERROR_NAMES
