@@ -7,9 +7,9 @@ differences between @dataclasses.dataclass and @chex.dataclass are:
 * Chex dataclasses have replace, from_tuple, and to_tuple methods.
 """
 
-from pytype import abstract
 from pytype import overlay
 from pytype import overlay_utils
+from pytype.abstract import abstract
 from pytype.overlays import classgen
 from pytype.overlays import dataclass_overlay
 from pytype.pytd import pytd
