@@ -57,6 +57,21 @@ class Annotated(abstract.AnnotationClass):
     return inner[0]
 
 
+class Final(abstract.AnnotationClass):
+  """Implementation of typing.Final[T]."""
+
+  def _build_value(self, node, inner, ellipses):
+    self.ctx.errorlog.invalid_ellipses(self.ctx.vm.frames, ellipses, self.name)
+    if len(inner) != 1:
+      error = "typing.Final must wrap a single type"
+      self.ctx.errorlog.invalid_annotation(self.ctx.vm.frames, self, error)
+    return abstract.FinalAnnotation(inner[0], self.ctx)
+
+  def instantiate(self, node, container=None):
+    self.ctx.errorlog.invalid_final_type(self.ctx.vm.frames)
+    return self.ctx.new_unsolvable(node)
+
+
 class TypingContainer(abstract.AnnotationContainer):
 
   def __init__(self, name, ctx):
@@ -222,6 +237,9 @@ class NoReturn(abstract.Singleton):
 
   def __init__(self, ctx):
     super().__init__("NoReturn", ctx)
+    # Sets cls to Type so that runtime usages of NoReturn don't cause pytype to
+    # think that NoReturn is being used illegally in type annotations.
+    self.cls = ctx.convert.type_type
 
 
 def build_any(ctx):
@@ -294,6 +312,28 @@ class Overload(abstract.PyTDFunction):
     return node, func_var
 
 
+class FinalDecorator(abstract.PyTDFunction):
+  """Implementation of typing.final."""
+
+  def call(self, node, unused_func, args):
+    """Marks that the given function is final."""
+    self.match_args(node, args)
+    arg = args.posargs[0]
+    for obj in arg.data:
+      if self._can_be_final(obj):
+        obj.final = True
+      else:
+        self.ctx.errorlog.bad_final_decorator(self.ctx.vm.frames, obj)
+    return node, arg
+
+  def _can_be_final(self, obj):
+    if isinstance(obj, abstract.Class):
+      return True
+    if isinstance(obj, abstract.Function):
+      return obj.is_method
+    return False
+
+
 class Generic(TypingContainer):
   """Implementation of typing.Generic."""
 
@@ -386,16 +426,20 @@ def build_cast(ctx):
   return Cast.make("cast", ctx, "typing")
 
 
+def build_final_decorator(ctx):
+  return FinalDecorator.make("final", ctx, "typing")
+
+
 def build_final(ctx):
-  ctx.errorlog.not_supported_yet(ctx.vm.frames, "typing.final")
-  return ctx.convert.name_to_value("typing.final")
+  return not_supported_yet("Final", ctx)
 
 
 typing_overlay = {
     "Annotated": overlay.build("Annotated", Annotated),
     "Any": build_any,
     "Callable": overlay.build("Callable", Callable),
-    "final": build_final,
+    "final": build_final_decorator,
+    "Final": overlay.build("Final", Final),
     "Generic": overlay.build("Generic", Generic),
     "Literal": overlay.build("Literal", Literal),
     "NamedTuple": build_namedtuple,
