@@ -14,7 +14,7 @@ import sys
 
 from pytype import utils
 from pytype.pyi import parser
-from pytype.pytd import builtins
+from pytype.pytd import builtin_stubs
 from pytype.pytd import optimize
 from pytype.pytd import pytd_utils
 
@@ -53,7 +53,7 @@ def make_parser():
       help="Remove mutable parameters.")
   o.add_argument(
       "-V", "--python_version", type=str, action="store",
-      dest="python_version", default="3",
+      dest="python_version", default=None,
       help=("Python version to target (\"major.minor\", e.g. \"3.7\")"))
   o.add_argument(
       "--multiline-args", action="store_true",
@@ -65,30 +65,36 @@ def make_parser():
 def main():
   argument_parser = make_parser()
   opts = argument_parser.parse_args()
-  python_version = utils.version_from_string(opts.python_version)
+  if opts.python_version:
+    python_version = utils.version_from_string(opts.python_version)
+  else:
+    python_version = sys.version_info[:2]
   try:
     utils.validate_version(python_version)
   except utils.UsageError as e:
     sys.stderr.write("Usage error: %s\n" % utils.message(e))
     sys.exit(1)
 
+  options = parser.PyiOptions(python_version=python_version)
+
   with open(opts.input) as fi:
     sourcecode = fi.read()
     try:
-      parsed = parser.parse_string(sourcecode, filename=opts.input,
-                                   python_version=python_version)
+      parsed = parser.parse_string(
+          sourcecode, filename=opts.input, options=options)
     except parser.ParseError as e:
       sys.stderr.write(str(e))
       sys.exit(1)
 
   if opts.optimize:
-    parsed = optimize.Optimize(parsed,
-                               builtins.GetBuiltinsPyTD(),
-                               lossy=opts.lossy,
-                               use_abcs=opts.use_abcs,
-                               max_union=opts.max_union,
-                               remove_mutable=opts.remove_mutable,
-                               can_do_lookup=False)
+    parsed = optimize.Optimize(
+        parsed,
+        pytd_utils.Concat(*builtin_stubs.GetBuiltinsAndTyping(options)),
+        lossy=opts.lossy,
+        use_abcs=opts.use_abcs,
+        max_union=opts.max_union,
+        remove_mutable=opts.remove_mutable,
+        can_do_lookup=False)
 
   if opts.output is not None:
     out_text = pytd_utils.Print(parsed, opts.multiline_args)
