@@ -203,6 +203,7 @@ class MethodTypes:
 class MethodFlags:
   ABSTRACT = 1
   COROUTINE = 2
+  FINAL = 4
 
   @classmethod
   def abstract_flag(cls, is_abstract):  # pylint: disable=invalid-name
@@ -234,6 +235,10 @@ class Function(Node):
   def is_coroutine(self):
     return bool(self.flags & MethodFlags.COROUTINE)
 
+  @property
+  def is_final(self):
+    return bool(self.flags & MethodFlags.FINAL)
+
   def with_flag(self, flag, value):
     """Return a copy of self with flag set to value."""
     new_flags = self.flags | flag if value else self.flags & ~flag
