@@ -1,7 +1,7 @@
 """Basic tests for accessing typegraph metrics from Python."""
 import textwrap
 
-from pytype import analyze
+from pytype import context
 from pytype import errors
 from pytype import typegraph
 from pytype.tests import test_base
@@ -12,10 +12,10 @@ class MetricsTest(test_base.BaseTest):
   def setUp(self):
     super().setUp()
     self.errorlog = errors.ErrorLog()
-    self.vm = analyze.CallTracer(self.errorlog, self.options, self.loader)
+    self.ctx = context.Context(self.errorlog, self.options, self.loader)
 
   def run_program(self, src):
-    return self.vm.run_program(textwrap.dedent(src), "", maximum_depth=10)
+    return self.ctx.vm.run_program(textwrap.dedent(src), "", maximum_depth=10)
 
   def assertNotEmpty(self, container, msg=None):
     if not container:
@@ -28,7 +28,7 @@ class MetricsTest(test_base.BaseTest):
           return x + 1
         a = foo(1)
     """)
-    metrics = self.vm.program.calculate_metrics()
+    metrics = self.ctx.program.calculate_metrics()
     # No specific numbers are used to prevent this from being a change detector.
     self.assertIsInstance(metrics, typegraph.cfg.Metrics)
     self.assertGreater(metrics.binding_count, 0)
